"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const slash_1 = __importDefault(require("slash"));
const defaultRequestToFile = (publicPath, root) => path_1.default.join(root, publicPath.slice(1));
const defaultFileToRequest = (filePath, root) => `/${slash_1.default(path_1.default.relative(root, filePath))}`;
const defaultIdToRquest = (id) => {
    if (id.startsWith('@') && id.indexOf('/') < 0) {
        return `/${id}`;
    }
};
const queryRE = /\?.*$/;
const ensureJs = (id) => {
    const cleanId = id.replace(queryRE, '');
    if (!/\.\w+/.test(cleanId)) {
        const queryMatch = id.match(queryRE);
        const query = queryMatch ? queryMatch[0] : '';
        return cleanId + '.js' + query;
    }
    return id;
};
function createResolver(root, resolvers) {
    return {
        requestToFile: (publicPath) => {
            let resolved;
            for (const r of resolvers) {
                const filepath = r.requestToFile(publicPath, root);
                if (filepath) {
                    resolved = filepath;
                    break;
                }
            }
            if (!resolved) {
                resolved = defaultRequestToFile(publicPath, root);
            }
            // @ is reserved for special modules, leave as-is
            if (resolved.startsWith(`/@`)) {
                return resolved;
            }
            else {
                return ensureJs(resolved);
            }
        },
        fileToRequest: (filePath) => {
            for (const r of resolvers) {
                const request = r.fileToRequest(filePath, root);
                if (request)
                    return request;
            }
            return defaultFileToRequest(filePath, root);
        },
        idToRequest: (id) => {
            for (const r of resolvers) {
                const request = r.idToRequest && r.idToRequest(id);
                if (request)
                    return request;
            }
            return defaultIdToRquest(id);
        }
    };
}
exports.createResolver = createResolver;
